import { openWindow } from '..';
import { dataURLtoBlob, urlToBase64 } from './base64Conver';
import { useMessage } from '/@/hooks/web/useMessage';

/**
 * Download online pictures
 * @param url
 * @param filename
 * @param mime
 * @param bom
 */
export function downloadByOnlineUrl(url: string, filename: string, mime?: string, bom?: BlobPart) {
  urlToBase64(url).then((base64) => {
    downloadByBase64(base64, filename, mime, bom);
  });
}

/**
 * Download pictures based on base64
 * @param buf
 * @param filename
 * @param mime
 * @param bom
 */
export function downloadByBase64(buf: string, filename: string, mime?: string, bom?: BlobPart) {
  const base64Buf = dataURLtoBlob(buf);
  downloadByData(base64Buf, filename, mime, bom);
}

/**
 * Download according to the background interface file stream
 * @param {*} data
 * @param {*} filename
 * @param {*} mime
 * @param {*} bom
 */
export function downloadByData(data: BlobPart, filename: string, mime?: string, bom?: BlobPart) {
  const blobData = typeof bom !== 'undefined' ? [bom, data] : [data];
  const blob = new Blob(blobData, { type: mime || 'application/octet-stream' });

  const blobURL = window.URL.createObjectURL(blob);
  const tempLink = document.createElement('a');
  tempLink.style.display = 'none';
  tempLink.href = blobURL;
  tempLink.setAttribute('download', filename);
  if (typeof tempLink.download === 'undefined') {
    tempLink.setAttribute('target', '_blank');
  }
  document.body.appendChild(tempLink);
  tempLink.click();
  document.body.removeChild(tempLink);
  window.URL.revokeObjectURL(blobURL);
}

/**
 * Download file according to file address
 * @param {*} sUrl
 */
export function downloadByUrl({
  url,
  target = '_blank',
  fileName,
}: {
  url: string;
  target?: TargetContext;
  fileName?: string;
}): boolean {
  const x = new window.XMLHttpRequest();
  x.open('GET', url, true);
  x.responseType = 'blob';
  x.onload = () => {
    let fileUrl = window.URL.createObjectURL(x.response);
    const isChrome = window.navigator.userAgent.toLowerCase().indexOf('chrome') > -1;
    const isSafari = window.navigator.userAgent.toLowerCase().indexOf('safari') > -1;

    if (/(iP)/g.test(window.navigator.userAgent)) {
      console.error('Your browser does not support download!');
      return false;
    }
    if (isChrome || isSafari) {
      const link = document.createElement('a');
      link.href = fileUrl;
      link.target = target;

      if (link.download !== undefined) {
        link.download = fileName || fileUrl.substring(fileUrl.lastIndexOf('/') + 1, fileUrl.length);
      }

      if (document.createEvent) {
        const e = document.createEvent('MouseEvents');
        e.initEvent('click', true, true);
        link.dispatchEvent(e);
        return true;
      }
    }
    if (fileUrl.indexOf('?') === -1) {
      fileUrl += '?download';
    }
    openWindow(fileUrl, { target });
  };
  x.send();
  return true;
}

/**
 * 下载文件到本地
 * @param blob 文件流
 * @param fileName 存储文件名称
 */
export const downloadFileByBlob = (blob: Blob, fileName?: string) => {
  const blobUrl = window.URL.createObjectURL(blob);
  const link = document.createElement('a');
  link.download = fileName ?? `下载文件_${new Date().getTime()}`;
  link.style.display = 'none';
  link.href = blobUrl;
  document.body.appendChild(link);
  link.click();
  document.body.removeChild(link);
};

const { createMessage } = useMessage();

export const baseDownloadFileUrl = (url, fileName) => {
  // val是标题 、url 是地址、fileName区分批量还是单个下载参数
  const aUrl = `${url}`;
  const x = new XMLHttpRequest();
  x.open('GET', aUrl, true);
  x.responseType = 'blob';
  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  x.onload = function () {
    const url = window.URL.createObjectURL(x.response);
    const elink = document.createElement('a');
    elink.href = url;
    elink.target = '_self'; // 当前也 target打开新页面
    // elink.setAttribute('download', fileName);
    elink.download = `${fileName}`;
    elink.style.display = 'none';
    document.body.appendChild(elink);
    createMessage.success(`${fileName} 下载成功`);
    setTimeout(() => {
      elink.click();
      document.body.removeChild(elink);
    }, 66);
  };
  x.send();
};
